###
# Application attachments class
# Copyright © 2017 Alexander Konov. All rights reserved.
###

'use strict'

DI = require './di/DI'

Field = require './field/Field'
merge = require './utils/merge'

class Attachments

  defaultOptions:
    dialog: require './utils/Dialog'
    animationSpeed: 500
    url:
      list: null
      upload: null
      delete: null
      find: null
    view:
      field: require './views/field/field'
      selector: require './views/field/selector'
      selectorFile: require './views/field/selectorFile'
    valueDelimiter: ', '
    extensions: null
    maxSelected: null
    maxUploadSize: null
    listLimit: 25
    schemes:
      requests:
        find:
          id: 'id'
          name: 'name'
          extension: 'extension'
        list:
          id: 'id'
          name: 'name'
          extension: 'extension'
          offset: 'offset'
          limit: 'limit'
        upload:
          file: 'file'
        delete:
          id: 'id'
      responses:
        file:
          id: 'id'
          name: 'name'
          extension: 'extension'
          size: 'size'
          url: 'url'
          preview_url: 'preview_url'
          created_at: 'created_at'
          updated_at: 'updated_at'
        list:
          total: 'total'
          files: 'files'
        upload:
          files: 'files'
        find:
          files: 'files'
    translate: null

  ###
  # Constructor
  # @param jQuery $ - jQuery instance
  # @param DOM el - an element which is attached to attachment field
  # @param {} options - object with options which configure Server object
  #  see description options in docs
  # @param DOM rootElement - element for storage rendered views
  ###
  constructor: ($, el, options, rootElement) ->
    @_$el = $ el

    @options = merge @defaultOptions, options

    for own name, source of @options.view
      if source.selector isnt undefined
        @options.view[name] = $(source.selector).html()

    @options.translate = {} unless @options.translate

    @_di = new DI()
    @_di
      .set '$', $
      .set 'lodash.template', require 'lodash.template'

      .set 'models.File', require './models/File'
      .set 'collections.FilesCollection', require './collections/FilesCollection'

      .set 'field.Selector', require './field/Selector'

      .set 'server.Server', require './server/Server'
      .set 'server.schemes.responses.File', require './server/schemes/responses/File'
      .set 'server.schemes.responses.Find', require './server/schemes/responses/Find'
      .set 'server.schemes.responses.List', require './server/schemes/responses/List'
      .set 'server.schemes.responses.Upload', require './server/schemes/responses/Upload'
      .set 'server.schemes.requests.Find', require './server/schemes/requests/Find'
      .set 'server.schemes.requests.List', require './server/schemes/requests/List'
      .set 'server.schemes.requests.Upload', require './server/schemes/requests/Upload'
      .set 'server.schemes.requests.Delete', require './server/schemes/requests/Delete'

      .set 'events.Events', require './events/Events'

      .set 'utils.DataMap', require './utils/DataMap'
      .set 'utils.Translate', require './utils/Translate'
      .set 'utils.Template', require './utils/Template'
      .set 'utils.uniqueID', require './utils/uniqueID'

      .setInstance 'collections.FilesCollection', [@_di]
      .setInstance 'events.Events', [@_di]
      .setInstance 'server.Server', [@_di, schemes: @options.schemes]
      .setInstance 'utils.Translate', [@options.translate]
      .setInstance 'utils.Template', [@_di]

    @_field = new Field @_di, el, @options, rootElement
    @_bind()

  ###
  # Bind to Field events
  # @return void
  ###
  _bind: ->
    events = @_di.getInstance 'events.Events'
    events.on @_field, 'change', (e, files)=>
      events.trigger @_$el, 'change', [@_filesToFields files]
    return


  ###
  # Get all files which loaded from server on current moment
  # @return File[]
  ###
  allReceivedFiles: ->
    return @_filesToFields @_field.allReceivedFiles()

  ###
  # Get all selected files
  # @return File[]
  ###
  selectedFiles: ->
    return @_filesToFields @_field.getSelectedFiles()

  ###
  # Select file by id
  # @param string in - file id for attachment
  # @return void
  ###
  selectFile: (id) ->
    @_field.setSelectedFiles [id], true
    return

  ###
  # Deselect file by id
  # @param string id - id file to deselect
  # @return void
  ###
  deSelectFile: (id) ->
    @_field.deSelectFile id
    return

  ###
  # Get unique id for current field
  # @return string
  ###
  getUID: () ->
    @_field.getUID()

  ###
  # Convert array with File objects to array with fields objects
  # @param File[] files - files for convert
  # @return Object[]
  ###
  _filesToFields: (files) ->
    (files[i].getFields() for i in [0...files.length])

module.exports = Attachments