###
# Dependency manager
# Copyright © 2017 Alexander Konov. All rights reserved.
###

'use strict'

class DI

  ###
  # Constructor
  ###
  constructor: ->
    @_functions = {}
    @_instancesData = {}
    @_instances = {}

  ###
  # Set dependency
  # @param string key - key for dependency
  # @param constructor|callable func - dependency
  ###
  set: (key, func) ->
    @_functions[key] = func
    @deleteInstance key
    @

  ###
  # Get dependency by key
  # @param string key - key for dependency
  # @return constructor|callable
  # @throw Error - error if dependency is not set
  ###
  get: (key) ->
    throw new Error 'Dependency with key ' + key + ' is not resolve.' unless @_functions[key]
    @_functions[key]

  ###
  # Delete dependency by key
  # @param string key - key for dependency
  ###
  delete: (key) ->
    delete @_functions[key] if @_functions[key]
    @

  ###
  # Create object on base dependency
  # @param string key - key for dependency
  # @param mixed[] args - arguments for dependency constructor
  # @return constructor|callable
  # @throw Error - error if dependency is not set or dependency is not constructor
  ###
  createObject: (key, args) ->
    args = args || []
    func = @get key
    new func(args...)

  ###
  # Set instance on base dependency
  # @param string key - key for dependency
  # @param mixed[] args - arguments for dependency constructor
  # @return this
  ###
  setInstance: (key, args) ->
    @_instancesData[key] = args || null
    @

  ###
  # Get dependency instance by key
  # @param string key - key for dependency
  # @return Object
  # @throw Error - error if dependency or dependency instance is not set
  ###
  getInstance: (key) ->
    return @_instances[key] if @_instances[key]
    throw new Error 'Instance data for dependency with key ' + key + ' is not set.' if @_instancesData[key] is undefined
    @_instances[key] = @createObject key, @_instancesData[key]
    @_instances[key]

  ###
  # Delete dependency instance by key
  # @param string key - key for dependency
  ###
  deleteInstance: (key) ->
    delete @_instancesData[key] if @_instancesData[key]
    delete @_instances[key] if @_instances[key]
    @

  ###
  # Check resolved required dependencies
  # @param string|string[] dependencyKeys - keys of required dependencies
  # @param string|string[] instanceKeys = [] - keys of required instances
  # @throw Error - error if dependencies is not resolve
  ###
  resolve: (dependencyKeys, instanceKeys) ->
    dependencyKeys = [dependencyKeys] unless dependencyKeys instanceof Array
    instanceKeys = [] unless instanceKeys
    instanceKeys = [instanceKeys] unless instanceKeys instanceof Array
    resolved = true
    for key in dependencyKeys
      unless @_functions[key]
        resolved = false
        break
    if resolved
      for key in instanceKeys
        if @_instancesData[key] is undefined
          resolved = false
          break
    throw new Error 'Unresolved dependencies.' unless resolved
    @

module.exports = DI