###
# Class for attachments field
# Copyright © 2017 Alexander Konov. All rights reserved.
###

'use strict'

class Field

  ###
  # Constructor
  # @param DI di - dependency injection object
  # @param DOM el - an element which is attached to attachment field
  # @param {} options - object with options which configure Server object
  #  see description options in docs
  # @param DOM rootElement - element for storage rendered views
  ###
  constructor: (di, el, options, rootElement) ->
    di.resolve [
      '$', 'collections.FilesCollection', 'server.Server',
      'utils.Translate', 'utils.Template',
      'field.Selector', 'events.Events'
    ], [
      'collections.FilesCollection', 'server.Server',
      'utils.Translate', 'utils.Template', 'events.Events'
    ]

    #unique id for each self object
    @_uid = di.get('utils.uniqueID')()

    @_$ = di.get '$'
    @_filesCollection = di.getInstance 'collections.FilesCollection'
    @_server = di.getInstance 'server.Server'
    @_translate = di.getInstance 'utils.Translate'
    @_template = di.getInstance 'utils.Template'
    @_selector = di.createObject 'field.Selector', [di, rootElement, options, @_uid]
    @_events = di.getInstance 'events.Events'

    @_options = options
    @_$el = @_$(el)
    @_$rootElement = @_$ rootElement #element for rendering
    @_selected = {}

    @_init()

  ###
  # Get all files which loaded from server on now moment
  # @return File[]
  ###
  allReceivedFiles: () ->
    @_filesCollection.getFiles()

  ###
  # Set selected files for attachment
  # @param string[]|string filesId - id files for attachments
  # @param bool noClear - switch for clear/complete selected files before set
  # @return void
  ###
  setSelectedFiles: (filesId, noClear) ->
    @_selected = {} unless noClear
    filesId = [filesId] unless filesId instanceof Array
    for id in filesId
      @_selected[id] = true if @_filesCollection.has id
    @_render()
    return

  ###
  # Get all selected files
  # @return File[]
  ###
  getSelectedFiles: ->
    result = []
    for own id of @_selected
      file = @_filesCollection.getFile id
      result.push file if file
    result

  ###
  # Deselect file by id
  # @param string id - id file to deselect
  # @return void
  ###
  deSelectFile: (id) ->
    return unless @_selected and @_selected[id]
    delete @_selected[id]
    $file = @_$view.find '[jq-attachments-selected-files-file][data-id="' + id + '"]'
    $file.hide @_options.animationSpeed, =>
      if @_getSelectedFilesID().length
        $file.remove()
        @_setInputVal @_getSelectedFilesID()
      else
        @_render()
      @_triggerChange()
    return

  ###
  # Get unique id for current field
  # @return string
  ###
  getUID: ->
    @_uid

  ###
  # Initialize field
  # @return void
  ###
  _init: () ->
    @_bindToSelector()

    @_$el.hide()

    attached = @_options.attached || []
    if attached.length
      @_filesCollection.addFiles attached

    @_loadSelected()
    return

  ###
  # Load selected files which exists in @_el value
  # @return void
  ###
  _loadSelected: () ->
    values = (@_$el.val().split @_options.valueDelimiter).map (fileID) ->
      fileID.trim()

    unless values.length
      @_render()
      return

    filesId = values.filter (id) => !@_filesCollection.has id
    @_server.find @_options.url.find, {id: filesId}, (data)=>
      @_filesCollection.addFiles data.files
      for file in data.files
        @setSelectedFiles file.id, true
      @_render()
    return

  ###
  # Get selected file id
  # @return string[]
  ###
  _getSelectedFilesID: ->
    result = []
    for own id of @_selected
      result.push id
    result

  ###
  # Render view
  # @return void
  ###
  _render: () ->
    @_$view.remove() if @_$view isnt undefined

    html = @_template.render @_options.view.field,
      files: @getSelectedFiles()
    @_$view = @_$(html)

    @_setInputVal @_getSelectedFilesID()
    @_$el.after @_$view
    @_bind @_$view
    return

  ###
  # Bind events on instead elements in field
  # @return void
  ###
  _bind: ($el) ->
    @_events.on $el, 'click', (e)=>
      @_selector.open @_getSelectedFilesID()
      e.preventDefault()
      false
    , selector: '[jq-attachments-add]'

    @_events.on $el, 'click', (e)=>
      if @_options.dialog.confirm @_translate.t 'Are you sure deselect file?'
        $file = @_$(e.target).closest '[jq-attachments-selected-files-file]'
        $file.hide @_options.animationSpeed, =>
          @deSelectFile $file.attr 'data-id'
      e.preventDefault()
    , selector: '[jq-attachments-selected-files-file-remove]'
    return

  ###
  # Bind events on instead Selector in field
  # @return void
  ###
  _bindToSelector: ->
    @_events.on @_selector, 'change', (e, filesId)=>
      @setSelectedFiles filesId
      @_triggerChange()

    @_events.on @_selector, 'delete', (e, id)=>
      @deSelectFile id

  ###
  # Set input value for @_el
  # @return void
  ###
  _setInputVal: (filesId) ->
    @_$el.val filesId.join @_options.valueDelimiter
    return

  ###
  # Trigger change event
  # @return void
  ###
  _triggerChange: ->
    @_events.trigger @, 'change', [@getSelectedFiles()]
    return


module.exports = Field