###
# Class for attachments field
# Copyright © 2017 Alexander Konov. All rights reserved.
###

'use strict'

class Selector

  ###
  # Constructor
  # @param DI di - dependency injection object
  # @param DOM el - an element which is attached to attachment field
  # @param {} options - object with options which configure Server object
  #  see description options in docs
  # @param DOM rootElement - element for storage rendered views
  ###
  constructor: (di, rootElement, options, uid) ->
    di.resolve [
      '$', 'collections.FilesCollection', 'server.Server',
      'utils.Template', 'utils.Translate', 'events.Events'
    ], [
      'collections.FilesCollection', 'server.Server',
      'utils.Template', 'utils.Translate', 'events.Events'
    ]

    @_$ = di.get '$'
    @_filesCollection = di.getInstance 'collections.FilesCollection'
    @_server = di.getInstance 'server.Server'
    @_template = di.getInstance 'utils.Template'
    @_translate = di.getInstance 'utils.Translate'
    @_events = di.getInstance 'events.Events'

    @_$rootElement = @_$ rootElement
    @_options = options
    @_uid = uid
    @_total = 0
    @_offset = 0
    @_selected = []
    @_reverted = []

  ###
  # Open selector
  # @param string[] selected - id selected files
  # @return
  ###
  open: (selected) ->
    @_render() unless @_$view

    changed = selected.concat @_selected
    @_selected = selected

    if @_total > 0
      @_renderFiles 0, null, null, changed
      @_show()
    else
      @_show()
      @_loadFiles()
    return

  ###
  # Render selector
  # @return void
  ###
  _render: () ->
    return if @_$view isnt undefined
    html = @_template.render @_options.view.selector,
      accept: @_accept()
      uid: @_uid
    @_$view = @_$(html)
    @_$rootElement.append @_$view
    @_bind()
    return

  ###
  # Accept extensions (for render)
  # @return string
  ###
  _accept: () ->
    return '' unless @_options.extensions
    result = null
    for extension in @_options.extensions
      extension = extension.toLowerCase()
      if result
        result += ', .' + extension
      else
        result = '.' + extension
    result

  ###
  # Bind events for selector
  # @return void
  ###
  _bind: ()->
    @_events.on @_$view, 'click', (e)=>
      @_cancel()
      e.preventDefault()
    , selector: '[jq-attachments-window-close]'

    @_events.on @_$view, 'click', (e)=>
      $file = @_$(e.target).closest '[jq-attachments-file]'
      selected = $file.attr('data-selected') is 'true'
      unless selected
        values = @_getCurrentSelected()
        if @_options.maxSelected and @_options.maxSelected <= values.length
          @_options.dialog.alert @_translate.t 'Sorry, you can select not more than {maxSelected} file(s).',
            maxSelected: @_options.maxSelected
          e.preventDefault()
          return false
      @_reverted.push $file.attr 'data-id'
      $file.attr 'data-selected', (if selected then 'false' else 'true')
    , selector: '[jq-attachments-file-select]'

    @_events.on @_$view, 'click', (e)=>
      $file = @_$(e.target).closest '[jq-attachments-file]'
      @_deleteFile $file.attr 'data-id'
      e.preventDefault()
    , selector: '[jq-attachments-file-delete]'

    @_events.on @_$view, 'click', (e)=>
      @_apply()
      e.preventDefault()
    , selector: '[jq-attachments-apply]'

    @_events.on @_$view, 'click', (e)=>
      @_loadFiles()
      e.preventDefault()
    , selector: '[jq-attachments-files-more]'

    @_events.on @_$view, 'click', (e)=>
      @_uploadFile()
      e.preventDefault()
    , selector: '[jq-attachments-upload-send]'

    return

  ###
  # Check status of selected file
  # @param string id - file id
  # @return bool
  ###
  _isFileSelected: (id)->
    @_$.inArray(id, @_selected) isnt -1

  ###
  # Render list files
  # @param int offset - starting offset for rendering in files collection
  # @param bool clear - switch clear rendering file
  # @param bool prepend - switch inserted rendered files to begin
  # @param string[] changed - changed files at which be re-rendered
  # @return bool
  ###
  _renderFiles: (offset, clear, prepend, changed)->
    changed = changed || []
    files = @_filesCollection.getFiles()
    $container = @_$view.find '[jq-attachments-files]'
    $container.find '[jq-attachments-file]'.remove() if clear
    for i in [offset...files.length]
      file = files[i]
      continue unless file
      id = file.getField 'id'
      $oldFile = $container.find('[jq-attachments-file][data-id="' + id + '"]')
      if $oldFile.length > 0
        continue if @_$.inArray(id, changed) is -1 and @_$.inArray(id, @_reverted) is -1
        $oldFile.replaceWith @_getRenderedFile file
      else
        if prepend
          $container.prepend @_getRenderedFile file
        else
          $container.append @_getRenderedFile file
    return

  ###
  # Render file to html code
  # @param File file - file for rendering
  # @return string
  ###
  _getRenderedFile: (file) ->
    html = @_template.render @_options.view.selectorFile,
      file: file
      selected: @_isFileSelected file.getField 'id'
    @_$(html)

  ###
  # Apply actual status of select files
  # @return void
  ###
  _apply: ->
    @_reverted = []
    values = @_getCurrentSelected()
    if values.length isnt @_selected.length
      change = true
    else
      change = false
      for id in values
        unless @_$.inArray id, @_selected
          change = true
          break
    @_selected = values
    @_events.trigger @, 'change', [values] if change
    @_hide()
    return

  ###
  # Close Selector without saving actual status
  # @return void
  ###
  _cancel: () ->
    @_hide()
    return

  ###
  # Show Selector
  # @return void
  ###
  _show: ->
    @_$view.show @_options.animationSpeed
    return

  ###
  # Hide Selector
  # @return void
  ###
  _hide: ->
    @_$view.hide @_options.animationSpeed
    return

  ###
  # Get current selected files
  # @return string[]
  ###
  _getCurrentSelected: () ->
    $files = @_$view.find '[jq-attachments-file][data-selected="true"]'
    values = (@_$($files[i]).attr 'data-id' for i in [0...$files.length])
    values

  ###
  # Load files to Selector from server
  # @param callable onSuccess - callback for success load files
  # @return void
  # @throw Error
  ###
  _loadFiles: (onSuccess) ->
    throw new Error 'Url for list files is not set.' unless @_options.url.list
    @_startProgress()
    @_server.list @_options.url.list,
      offset: @_offset
      limit: @_options.listLimit
      extension: @_options.extensions
    , (data) =>
      @_filesCollection.addFiles data.files
      @_renderFiles @_offset
      @_offset += @_options.listLimit
      @_total = data.total
      if @_offset >= @_total
        @_$view.find('[jq-attachments-files-more]').hide()
      else
        @_$view.find('[jq-attachments-files-more]').show()
      onSuccess data if onSuccess
      @_stopProgress()
    , (xhr, status) =>
      @_stopProgress()
      throw new Error "Can not load files. Error code #{status}."

  ###
  # Delete file from server
  # @param string id - id of deleted file
  # @return void
  # @throw Error
  ###
  _deleteFile: (id) ->
    return unless @_options.dialog.confirm @_translate.t 'Are you sure want to delete the file?'
    @_startProgress()
    @_server.delete @_options.url.delete, id, =>
      @_filesCollection.removeFile id
      @_offset-- if @_offset
      $file = @_$view.find('[jq-attachments-file][data-id="' + id + '"]')
      @_events.trigger @, 'delete', [id]
      $file.hide @_options.animationSpeed, =>
        $file.remove()
        @_stopProgress()
    , (xhr, status) =>
      @_stopProgress()
      throw new Error "Can not delete file. Error code #{status}."

  ###
  # Upload files to server
  # @return void
  # @throw Error
  ###
  _uploadFile: ()->
    $file = @_$view.find '[jq-attachments-upload]'
    return unless $file.get(0).files.length
    formData = new FormData($file.closest('form').get(0))
    files = formData.getAll 'file[]'
    errors = []
    for file in files
      valid = @_validateFile file
      if valid isnt true
        errors.push @_translate.t 'File "{name}" is not valid. Error: "{error}"',
          name: file.name
          error: valid
    if errors.length > 0
      @_options.dialog.alert errors.join "\n"
      return

    @_startProgress()
    @_server.upload @_options.url.upload, formData, (data) =>
      $file
        .val ''
        .trigger 'change'
      offset = @_filesCollection.length()
      @_filesCollection.addFiles data.files
      @_renderFiles offset, false, true
      @_stopProgress()
    , (xhr, status) =>
      @_stopProgress()
      throw new Error "Can not upload files. Error code #{status}."

  ###
  # Validate file
  # @return true|sting - true if file is valid otherwise error message
  # @throw Error
  ###
  _validateFile: (file)->
    if @_options.maxUploadSize
      if @_options.maxUploadSize < file.size
        return @_translate.t 'File size should be <= {size} kb.', size: Math.round @_options.maxUploadSize / 1024, 2
    if @_options.extensions
      extension = file.name.replace /^.+\.([a-z0-9]+)$/, '$1'
      if @_$.inArray(extension, @_options.extensions) is -1
        return @_translate.t 'Invalid file extension. Allowed the following extensions: {extensions}.',
          extensions: @_options.extensions.join ', '
    true



  ###
  # Show progress line
  # @return void
  ###
  _startProgress: () ->
    @_$view.find('[jq-attachments-progress]').show() if @_$view

  ###
  # Hide progress line
  # @return void
  ###
  _stopProgress: ()->
    @_$view.find('[jq-attachments-progress]').hide() if @_$view

module.exports = Selector