###
# Class for make requests to server
# Copyright © 2017 Alexander Konov. All rights reserved.
###

'use strict'

class Server

  ###
  # Constructor
  # @param DI di - dependency injection object
  # @param {} options - object with options which configure Server object
  #  required
  #  options = {
  #    scheme: {
  #      file: fields for FileScheme,
  #      find: fields FindScheme,
  #      list: fields ListScheme,
  #      upload: fields UploadScheme,
  #    }
  #  }
  ###
  constructor: (di, options)->
    di.resolve [
      '$',
      'server.schemes.requests.Find', 'server.schemes.requests.List',
      'server.schemes.requests.Upload', 'server.schemes.requests.Delete',
      'server.schemes.responses.File', 'server.schemes.responses.Find',
      'server.schemes.responses.List', 'server.schemes.responses.Upload'
    ]

    @_$ = di.get '$'

    fileScheme = di.createObject 'server.schemes.responses.File', [options.schemes.responses.file]
    @_schemes =
      requests:
        find: di.createObject 'server.schemes.requests.Find', [options.schemes.requests.find]
        list: di.createObject 'server.schemes.requests.List', [options.schemes.requests.list]
        upload: di.createObject 'server.schemes.requests.Upload', [options.schemes.requests.upload]
        delete: di.createObject 'server.schemes.requests.Delete', [options.schemes.requests.delete]
      responses:
        file: fileScheme
        find: di.createObject 'server.schemes.responses.Find', [options.schemes.responses.find, files: fileScheme]
        list: di.createObject 'server.schemes.responses.List', [options.schemes.responses.list, files: fileScheme]
        upload: di.createObject 'server.schemes.responses.Upload', [options.schemes.responses.upload, files: fileScheme]


  ###
  # Send request for find files
  # @param string url - request url
  # @param mixed[] data - request data (sending on server how GET parameters)
  # @param callable onSuccess - callback which invoke done request is success
  #    have following parameters:
  #      mixed{} result - data from server as json object
  # @param callable onError - callback which invoke done request is fail
  #    have following parameters:
  #      jqXHR jqXHR - object jQuery XMLHttpRequest
  #      string textStatus - textStatus of error
  ###
  find: (url, data, onSuccess, onError) ->
    @_$.ajax(
      url: url
      type: 'GET'
      dataType: 'json'
      data: @_schemes.requests.find.toScheme data
    ).done((result) =>
      data = @_schemes.responses.find.toData result
      onSuccess data if onSuccess
    ).fail((jqXHR, textStatus) =>
      onError jqXHR, textStatus if onError
    )
  ###
  # Send request for list files
  # @param string url - request url
  # @param mixed[] data - request data (sending on server how GET parameters)
  # @param callable onSuccess - callback which invoke done request is success
  #    have following parameters:
  #      mixed{} result - data from server as json object
  # @param callable onError - callback which invoke done request is fail
  #    have following parameters:
  #      jqXHR jqXHR - object jQuery XMLHttpRequest
  #      string textStatus - textStatus of error
  ###
  list: (url, params, onSuccess, onError) ->
    @_$.ajax(
      url: url
      type: 'GET'
      dataType: 'json'
      data: @_schemes.requests.list.toScheme params
    ).done((data) =>
      data = @_schemes.responses.list.toData data
      onSuccess data if onSuccess
    ).fail((jqXHR, textStatus) =>
      onError jqXHR, textStatus if onError
    )

  ###
  # Upload files on server
  # @param string url - request url
  # @param FormData formData - object FormData which contain files for uploading
  # @param callable onSuccess - callback which invoke done request is success
  #    have following parameters:
  #      mixed{} result - data from server as json object
  # @param callable onError - callback which invoke done request is fail
  #    have following parameters:
  #      jqXHR jqXHR - object jQuery XMLHttpRequest
  #      string textStatus - textStatus of error
  ###
  upload: (url, formData, onSuccess, onError) ->
    data = new FormData()
    paramName = @_schemes.requests.upload.getSchemeFieldName 'file'
    files = formData.getAll 'file[]'
    for file in files
      data.append paramName+'[]', file
    @_$.ajax(
      url: url
      type: 'POST'
      contentType: false
      processData: false
      dataType: 'json'
      data: data
    ).done((data) =>
      data = @_schemes.responses.upload.toData data
      onSuccess data if onSuccess
    ).fail((jqXHR, textStatus) =>
      onError jqXHR, textStatus if onError
    )

  ###
  # Send request for delete file
  # @param string url - request url
  # @param mixed[] data - request data (sending on server how GET parameters)
  # @param callable onSuccess - callback which invoke done request is success
  #    have following parameters:
  #      no params
  # @param callable onError - callback which invoke done request is fail
  #    have following parameters:
  #      jqXHR jqXHR - object jQuery XMLHttpRequest
  #      string textStatus - textStatus of error
  ###
  delete: (url, id, onSuccess, onError) ->
    @_$.ajax(
      url: url
      type: 'GET'
      dataType: 'json'
      data: @_schemes.requests.delete.toScheme id: id
    ).done(() =>
      onSuccess() if onSuccess
    ).fail((jqXHR, textStatus) =>
      onError jqXHR, textStatus if onError
    )

module.exports = Server