###
# Abstract scheme to associate fields from server with fields from model
# Copyright © 2017 Alexander Konov. All rights reserved.
###

'use strict'

class AbstractScheme

  @FORMAT_TYPE_DATA = 'data'
  @FORMAT_TYPE_SCHEME = 'scheme'

  @DEPENDENCY_TYPE_ARRAY = 'array'
  @DEPENDENCY_TYPE_OBJECT = 'object'

  ###
  # Constructor
  # @param string{} fields - object with fields and aliases from server
  #   format: {fieldName:serverAliasName, }
  #   example: {id: 'server_model_id', name: 'full_name', size: 'count'}
  # @param AbstractScheme{} dependencies - objects with dependencies
  #   format: {dependencyName: dependencyObject, }
  ###
  constructor: (fields, dependencies)->
    @_fields = fields
    @_dependencies = if dependencies then dependencies else {}
    throw new Error('Incorrect fields.') unless @_checkFields()
    throw new Error('Incorrect dependencies.') unless @_checkDependencies()

  ###
  # Abstract method
  # Return array with required fields
  # @return string[]
  ###
  fields: ->
    []

  ###
  # Abstract method
  # Return array with required dependencies
  # @return {fieldName:class, ...} || {fieldName: ['type', class], ...}
  ###
  dependencies: ->
    {}

  ###
  # Associate fields from server to fields from model
  # @param mixed{} schemeFields - scheme data
  #   example: toData({user_id: 1, user_name: 'mr. Smith'}) returned {id: 1, name: 'mt. Smith'}
  #            where this.fields() == ['id', 'name'] && this._fields = {id:'user_id', name: 'user_name'} (see constructor)
  ###
  toData: (schemeFields)->
    result = {}
    for own dataKey, schemeKey of @_fields
      value = schemeFields[schemeKey]
      continue if value is undefined
      result[dataKey] = @_formatValue @constructor.FORMAT_TYPE_DATA, dataKey, value
    result

  ###
  # Associate fields from model to fields from server
  # @param mixed{} dataFields - local data
  #   Example:
  #            toScheme({id: 1, name: 'mt. Smith'}) returned {user_id: 1, user_name: 'mr. Smith'}
  #            where this.fields() == ['id', 'name'] && this._fields = {id:'user_id', name: 'user_name'} (see constructor)
  ###
  toScheme: (dataFields) ->
    result = {}
    for own dataKey, schemeKey of @_fields
      value = dataFields[dataKey]
      continue if value is undefined
      result[schemeKey] = @_formatValue @constructor.FORMAT_TYPE_SCHEME, dataKey, value
    result

  ###
  # Get scheme field name by data field name
  # @params string field - data field name
  # @return string|undefined
  ###
  getSchemeFieldName: (field) ->
    return undefined if @_fields[field] is undefined
    @_fields[field]

  ###
  # Check on required fields
  # @return bool
  ###
  _checkFields: ->
    result = true
    fields = @fields()
    for field in fields
      if @_fields[field] is undefined
        result = false
        break
    result

  ###
  # Check on required dependencies
  # @return bool
  ###
  _checkDependencies: ->
    dependencies = @dependencies()
    for own field, dependency of dependencies
      value = @_dependencies[field]
      unless dependency instanceof Array
        return false unless value instanceof dependency
      else
        dependency = dependency[1]
        return false unless value instanceof dependency
    true


  ###
  # Return field value for this scheme
  # @param string type - type of format (see const FORMAT_TYPE_*)
  # @param string dataKey - key field for local model
  # @param mixed value - input value for associating
  # @return mixed
  ###
  _formatValue: (type, dataKey, value) ->
    dependency = @_dependencies[dataKey]
    return value unless dependency
    dependencyConfig = @dependencies()[dataKey]
    return @_dependencyValue type, value, dependency  unless dependencyConfig instanceof Array

    dependencyType = dependencyConfig[0]
    if dependencyType is @constructor.DEPENDENCY_TYPE_ARRAY
      return undefined unless value instanceof Array
      result = []
      for valueItem in value
        result.push @_dependencyValue type, valueItem, dependency
      return result

    if dependencyType is @constructor.DEPENDENCY_TYPE_OBJECT
      result = {}
      for own valueKey of value
        result[valueKey] = @_dependencyValue type, value[valueKey], dependency
      return result

    null

  ###
  # Return dependency value according to type
  # @param string type - type of format (see const FORMAT_TYPE_*)
  # @param mixed value - input value for associating
  # @param AbstractScheme dependency - dependency object for this value
  # @return mixed
  ###
  _dependencyValue: (type, value, dependency) ->
    if type is @constructor.FORMAT_TYPE_DATA
      return dependency.toData value
    if type is @constructor.FORMAT_TYPE_SCHEME
      return dependency.toScheme value
    undefined


module.exports = AbstractScheme