###
# Test for abstract server scheme
# Copyright © 2017 Alexander Konov. All rights reserved.
###

'use strict'

AbstractScheme = require '../../../src/server/schemes/AbstractScheme'

describe 'server.schemes.AbstractScheme', ->
  it 'should be create with corrected fields', ->
    class TestScheme extends AbstractScheme
      fields: ->
        [
          'id',
          'f1',
          'f2'
        ]

    expect(-> new TestScheme id: 'scheme_id', f1: 'scheme_f1', f2: 'scheme_f2').not.toThrow()
    return

  ######################################################################################################################

  it 'should be set throw exception when create scheme without required field', ->
    class TestScheme extends AbstractScheme
      fields: ->
        [
          'id',
          'f1',
          'f2'
        ]

    expect(-> new TestScheme id: 'scheme_id', f1: 'scheme_f1').toThrowError 'Incorrect fields.'
    return

  ######################################################################################################################

  it 'should be create with correct dependencies', ->
    class DependencyScheme extends AbstractScheme
      fields: ->
        [
          'id',
          'f1',
          'f2'
        ]
    class TestScheme extends AbstractScheme
      fields: ->
        [
          'id',
          'd_o',
          'd_a'
        ]
      dependencies: ->
        {
          d_o: [AbstractScheme.DEPENDENCY_TYPE_OBJECT, DependencyScheme]
          d_a: [AbstractScheme.DEPENDENCY_TYPE_ARRAY, DependencyScheme]
        }
    dependency = new DependencyScheme id: 'id', f1: 'f1', f2: 'f2'

    fn = -> new TestScheme {
      id: 'id'
      d_o: 'd_o'
      d_a: 'd_a'
    }, {
      d_o: dependency
      d_a: dependency
    }

    expect(fn).not.toThrow()
    return

  ######################################################################################################################

  it 'should be set throw exception when create scheme without required dependency', ->
    class DependencyScheme extends AbstractScheme
      fields: ->
        [
          'id',
          'f1',
          'f2'
        ]
    class TestScheme extends AbstractScheme
      fields: ->
        [
          'id',
          'd_o',
          'd_a'
        ]
      dependencies: ->
        {
          d_o: [AbstractScheme.DEPENDENCY_TYPE_OBJECT, DependencyScheme]
          d_a: [AbstractScheme.DEPENDENCY_TYPE_ARRAY, DependencyScheme]
        }
    dependency = new DependencyScheme id: 'id', f1: 'f1', f2: 'f2'

    fn = -> new TestScheme {
      id: 'id'
      d_o: 'd_o'
      d_a: 'd_a'
    }, {
      d_a: dependency
    }
    expect(fn).toThrowError 'Incorrect dependencies.'

    fn = -> new TestScheme {
      id: 'id'
      d_o: 'd_o'
      d_a: 'd_a'
    }, {
      d_o: dependency
    }
    expect(fn).toThrowError 'Incorrect dependencies.'
    return

  ######################################################################################################################

  it 'should be convert data scheme to data model', ->
    class DependencyScheme extends AbstractScheme
      fields: ->
        [
          'id',
          'f1'
        ]
    class TestScheme extends AbstractScheme
      fields: ->
        [
          'id',
          'd_o',
          'd_a'
        ]
      dependencies: ->
        {
          d_o: [AbstractScheme.DEPENDENCY_TYPE_OBJECT, DependencyScheme]
          d_a: [AbstractScheme.DEPENDENCY_TYPE_ARRAY, DependencyScheme]
        }
    dependency = new DependencyScheme id: 's_id', f1: 's_f1'

    scheme = new TestScheme {
      id: 's_id'
      d_o: 's_d_o'
      d_a: 's_d_a'
    }, {
      d_o: dependency
      d_a: dependency
    }

    schemeData =
      s_id: 1
      s_d_o:
        1:
          s_id: 1
          s_f1: 'value_1'
        2:
          s_id: 2
          s_f1: 'value_2'
      s_d_a: [
        s_id: 3
        s_f1: 'value_3'
      ,
        s_id: 4
        s_f1: 'value_4'
      ]

    modelData =
      id: 1
      d_o:
        1:
          id: 1
          f1: 'value_1'
        2:
          id: 2
          f1: 'value_2'
      d_a: [
        id: 3
        f1: 'value_3'
      ,
        id: 4
        f1: 'value_4'
      ]

    expect(scheme.toData(schemeData)).toEqual(modelData)
    return

  ######################################################################################################################

  it 'should be convert data model to data scheme', ->
    class DependencyScheme extends AbstractScheme
      fields: ->
        [
          'id',
          'f1'
        ]
    class TestScheme extends AbstractScheme
      fields: ->
        [
          'id',
          'd_o',
          'd_a'
        ]
      dependencies: ->
        {
          d_o: [AbstractScheme.DEPENDENCY_TYPE_OBJECT, DependencyScheme]
          d_a: [AbstractScheme.DEPENDENCY_TYPE_ARRAY, DependencyScheme]
        }
    dependency = new DependencyScheme id: 's_id', f1: 's_f1'

    scheme = new TestScheme {
      id: 's_id'
      d_o: 's_d_o'
      d_a: 's_d_a'
    }, {
      d_o: dependency
      d_a: dependency
    }

    schemeData =
      s_id: 1
      s_d_o:
        1:
          s_id: 1
          s_f1: 'value_1'
        2:
          s_id: 2
          s_f1: 'value_2'
      s_d_a: [
        s_id: 3
        s_f1: 'value_3'
      ,
        s_id: 4
        s_f1: 'value_4'
      ]

    modelData =
      id: 1
      d_o:
        1:
          id: 1
          f1: 'value_1'
        2:
          id: 2
          f1: 'value_2'
      d_a: [
        id: 3
        f1: 'value_3'
      ,
        id: 4
        f1: 'value_4'
      ]

    expect(scheme.toScheme(modelData)).toEqual(schemeData)
    return

  ######################################################################################################################

  it 'should be convert data scheme with no all required fields to data model', ->
    class DependencyScheme extends AbstractScheme
      fields: ->
        [
          'id',
          'f1'
        ]
    class TestScheme extends AbstractScheme
      fields: ->
        [
          'id',
          'd_o',
          'd_a'
        ]
      dependencies: ->
        {
          d_o: [AbstractScheme.DEPENDENCY_TYPE_OBJECT, DependencyScheme]
          d_a: [AbstractScheme.DEPENDENCY_TYPE_ARRAY, DependencyScheme]
        }
    dependency = new DependencyScheme id: 's_id', f1: 's_f1'

    scheme = new TestScheme {
      id: 's_id'
      d_o: 's_d_o'
      d_a: 's_d_a'
    }, {
      d_o: dependency
      d_a: dependency
    }

    schemeData =
      s_id: 1
      s_d_o:
        1:
          s_id: 1
        2:
          s_id: 2
          s_f1: 'value_2'

    modelData =
      id: 1
      d_o:
        1:
          id: 1
        2:
          id: 2
          f1: 'value_2'

    expect(scheme.toData(schemeData)).toEqual(modelData)
    return

  ######################################################################################################################

  it 'should be convert data model with not included fields to data scheme', ->
    class DependencyScheme extends AbstractScheme
      fields: ->
        [
          'id',
          'f1'
        ]
    class TestScheme extends AbstractScheme
      fields: ->
        [
          'id',
          'd_o',
          'd_a'
        ]
      dependencies: ->
        {
          d_o: [AbstractScheme.DEPENDENCY_TYPE_OBJECT, DependencyScheme]
          d_a: [AbstractScheme.DEPENDENCY_TYPE_ARRAY, DependencyScheme]
        }
    dependency = new DependencyScheme id: 's_id', f1: 's_f1'

    scheme = new TestScheme {
      id: 's_id'
      d_o: 's_d_o'
      d_a: 's_d_a'
    }, {
      d_o: dependency
      d_a: dependency
    }

    schemeData =
      s_id: 1
      s_d_o:
        1:
          s_id: 1
          s_f1: 'value_1'
        2:
          s_id: 2

    modelData =
      id: 1
      d_o:
        1:
          id: 1
          f1: 'value_1'
        2:
          id: 2

    expect(scheme.toScheme(modelData)).toEqual(schemeData)
    return

  it 'should be return corrected scheme field name', ->
    class TestScheme extends AbstractScheme
      fields: ->
        [
          'id',
        ]
    scheme = new TestScheme id:'test_id'

    expect(scheme.getSchemeFieldName('id')).toBe 'test_id'
